@@ -124,12 +124,224 @@ import matplotlib.pyplot as plt
 import pandas as pd
 import numpy as np
 from packaging import version
+import os
+from imagesc.utils.adjmat_vec import adjmat2vec
+from shutil import copyfile
+import webbrowser
+import tempfile
+curpath = os.path.dirname(os.path.abspath(__file__))
+
+
+# %%
+def d3(df, path=None, title='d3 Heatmap!', description='Heatmap description', width=500, height=500, fontsize=10, cmap='interpolateInferno', scale=False, vmin=None, vmax=None, showfig=True, stroke='red', verbose=3):
+    """Heatmap in d3 javascript.
+
+    Parameters
+    ----------
+    df : pd.DataFrame()
+        Input data. The index and column names are used for the row/column naming.
+    path : String, (Default: user temp directory)
+        Directory path to save the output, such as 'c://temp/index.html'
+    title : String, (default: 'd3 Heatmap!')
+        Title text.
+    description : String, (default: 'Heatmap description')
+        Description text of the heatmap.
+    width : int, (default: 500).
+        Width of the window.
+    height : int, (default: 500).
+        height of the window.
+    fontsize : int, (default: 10).
+        Font size for the X and Y labels.
+    scale : Bool, (default: True).
+        Scale the values between [0-100].
+    vmin : Bool, (default: 0).
+        Range of colors starting with minimum value.
+            * 1 : cells with value <1 are coloured white.
+            * None : cells are colored based on the minimum value in the input data.
+    vmax : Bool, (default: 100).
+        Range of colors starting with maximum value.
+            * 100 : cells above value >100 are capped.
+            * None : cells are colored based on the maximum value in the input data.
+    stroke : String, (default: 'red').
+        Color of the recangle when hovering over a cell.
+            * 'red'
+            * 'black'
+    showfig : Bool, (default: True)
+        Open browser with heatmap.
+    cmap : String, (default: 'interpolateInferno').
+        The colormap scheme. This can be found at: https://github.com/d3/d3-scale-chromatic.
+        Categorical:
+            * 'schemeCategory10'
+            * 'schemeAccent'
+            * 'schemeDark2'
+            * 'schemePaired'
+            * 'schemePastel1'
+            * 'schemePastel2'
+            * 'schemeSet1'
+            * 'schemeSet2'
+            * 'schemeSet3'
+            * 'schemeTableau10'
+        Diverging:
+            * 'interpolateInferno'
+            * 'interpolatePRGn'
+            * 'interpolatePiYG'
+            * 'interpolatePuOr'
+            * 'interpolateRdBu'
+            * 'interpolateRdGy'
+            * 'interpolateRdYlBu'
+            * 'interpolateRdYlGn'
+            * 'interpolateSpectral'
+        Single color:
+            * 'interpolateBlues'
+            * 'interpolateGreens'
+            * 'interpolateGreys'
+            * 'interpolateOranges'
+            * 'interpolatePurples'
+            * 'interpolateReds'
+        Sequential:
+            * 'interpolateTurbo'
+            * 'interpolateViridis'
+            * 'interpolateInferno'
+            * 'interpolateMagma'
+            * 'interpolatePlasma'
+            * 'interpolateCividis'
+            * 'interpolateWarm'
+            * 'interpolateCool'
+            * 'interpolateCubehelixDefault'
+            * 'interpolateBuGn'
+            * 'interpolateBuPu'
+            * 'interpolateGnBu'
+            * 'interpolateOrRd'
+            * 'interpolatePuBuGn'
+            * 'interpolatePuBu'
+            * 'interpolatePuRd'
+            * 'interpolateRdPu'
+            * 'interpolateYlGnBu'
+            * 'interpolateYlGn'
+            * 'interpolateYlOrBr'
+            * 'interpolateYlOrRd'
+        Cyclic:
+            * 'interpolateRainbow'
+            * 'interpolateSinebow'
+    verbose : int [0-5], (default: 3)
+        Verbosity to print the working-status. The higher the number, the more information.
+            * 0: None
+            * 1: Error
+            * 2: Warning
+            * 3: Info
+            * 4: Debug
+            * 5: Trace
+
+    Example
+    -------
+    >>> import imagesc
+    >>> df = pd.DataFrame(np.random.randint(0, 10, size=(7, 52)))
+    >>> paths = imagesc.d3(df, fontsize=10, title='Python to d3 conversion!', description='Click here for the interactive website!', path='d3heatmap.html', height=200, width=750, cmap='interpolateGreens')
+
+    Returns
+    -------
+    out : dict.
+        output path names.
+
+    """
+    if cmap in ['schemeCategory10', 'schemeAccent', 'schemeDark2', 'schemePaired', 'schemePastel2', 'schemePastel1', 'schemeSet1', 'schemeSet2', 'schemeSet3', 'schemeTableau10']:
+        cmap_type='scaleOrdinal'
+        if verbose>=3: print('[imagesc] >d3 cmap type is set to %s' %(cmap_type))
+    else:
+        cmap_type='scaleSequential'
+
+    # Rescale data between 0-100
+    if scale:
+        df = _scale(df, verbose=verbose)
+    if (not scale) and (vmin is not None) and (vmax is not None):
+        if verbose>=3: print('[imagesc] >Data is not scaled. Tip: set vmin=None and vmax=None to range colors between min-max of your data.')
+    if vmin is None:
+        vmin = np.min(df.values)
+    if vmax is None:
+        vmax = np.max(df.values)
+    if verbose>=3: print('[imagesc] >vmin is set to: %g' %(vmin))
+    if verbose>=3: print('[imagesc] >vmax is set to: %g' %(vmax))
+
+    # Get path to files
+    d3_library = os.path.abspath(os.path.join(curpath, 'd3js/d3.v4.js'))
+    d3_chromatic = os.path.abspath(os.path.join(curpath, 'd3js/d3.scale.chromatic.v1.min.js'))
+    d3_script = os.path.abspath(os.path.join(curpath, 'd3js/d3script.html'))
+
+    # Set fontsize for x-axis, y-axis
+    fontsize_x = fontsize
+    fontsize_y = fontsize
+
+    # Check path
+    filename, dirpath, path = _path_check(path, verbose)
+
+    # Copy files to destination directory
+    copyfile(d3_library, os.path.join(dirpath, os.path.basename(d3_library)))
+    copyfile(d3_chromatic, os.path.join(dirpath, os.path.basename(d3_chromatic)))
+    copyfile(d3_script, path)
+
+    # Convert into adj into vector
+    dfvec = adjmat2vec(df)
+    dfvec = dfvec.rename(columns={'source': 'variable', 'target': 'group', 'weight': 'value'})
+
+    # Write to disk (file is not used)
+    basename, ext = os.path.splitext(filename)
+    PATHNAME_TO_CSV = os.path.join(dirpath, basename + '.csv')
+    dfvec.to_csv(PATHNAME_TO_CSV, index=False)
+
+    # Embed the Data in the HTML. Note that the embedding is an important stap te prevent security issues by the browsers.
+    # Most (if not all) browser do not accept to read a file using d3.csv or so. It then requires security-by-passes, but thats not the way to go.
+    # An alternative is use local-host and CORS but then the approach is not user-friendly coz setting up this, is not so straightforward.
+    # It leaves us by embedding the data in the HTML. Thats what we are going to do here.
+    DATA_STR = ''
+    for i in range(0, dfvec.shape[0]):
+        newline = '{group : "' + str(dfvec['group'].iloc[i]) + '", variable : "' + str(dfvec['variable'].iloc[i]) + '", value : "' + str(dfvec['value'].iloc[i]) +'"},'
+        newline = newline + '\n'
+        DATA_STR = DATA_STR + newline
+
+    # Read the data
+    # var data =
+    # 	[
+    # 		{"group":"A", "variable":"v1", "value":"3"},
+    # 		{"group":"A", "variable":"v2", "value":"5"},
+    # 		{"group":"B", "variable":"v1", "value":"10"},
+    # 		{"group":"B", "variable":"v2", "value":"10"}
+    # 	]
+
+    # Import in the file
+    with open(path, 'r') as file: d3graphscript = file.read()
+
+    # Read the d3 html with script file
+    d3graphscript = d3graphscript.replace('$DESCRIPTION$', str(description))
+    d3graphscript = d3graphscript.replace('$TITLE$', str(title))
+
+    d3graphscript = d3graphscript.replace('$WIDTH$', str(width))
+    d3graphscript = d3graphscript.replace('$HEIGHT$', str(height))
+
+    d3graphscript = d3graphscript.replace('$VMIN$', str(vmin))
+    d3graphscript = d3graphscript.replace('$VMAX$', str(vmax))
+
+    d3graphscript = d3graphscript.replace('$FONTSIZE_X$', str(fontsize_x))
+    d3graphscript = d3graphscript.replace('$FONTSIZE_Y$', str(fontsize_y))
+
+    d3graphscript = d3graphscript.replace('$STROKE$', str(stroke))
+    d3graphscript = d3graphscript.replace('$CMAP$', str(cmap))
+    d3graphscript = d3graphscript.replace('$CMAP_TYPE$', str(cmap_type))
+
+    d3graphscript = d3graphscript.replace('$DATA_PATH$', filename)
+    d3graphscript = d3graphscript.replace('$DATA_COMES_HERE$', DATA_STR)
+
+    # Write to file
+    with open(path, 'w') as file: file.write(d3graphscript)
+    # Open browser with heatmap
+    if showfig: webbrowser.open(path, new=1)
+    # Return
+    out = {}
+    out['filename'] = filename
+    out['dirpath'] = dirpath
+    out['path'] = path
+    out['csv'] = PATHNAME_TO_CSV
+    return out
 
-# %% Adjust figure size based on input
-def _set_figsize(data_shape, figsize):
-    data_ratio = np.minimum(5/(data_shape[0]/data_shape[1]), 50)
-    out = tuple(np.ceil(np.interp(data_shape-np.min(data_shape), [np.min(figsize), data_ratio], [np.max(figsize), data_ratio])))
-    return(out[0], out[1])
 
 # %%
 def plot(data, row_labels=None, col_labels=None, **args):
@@ -152,7 +364,7 @@ def plot(data, row_labels=None, col_labels=None, **args):
 
 
     """
-    assert not isinstance(data, pd.DataFrame), print('[IMAGESC] data input must be numpy array')
+    assert not isinstance(data, pd.DataFrame), print('[imagesc] >data input must be numpy array')
     # Set defaults
     args, args_im = _defaults(args)
     # Set figsize based on data shape
@@ -163,6 +375,7 @@ def plot(data, row_labels=None, col_labels=None, **args):
     data = _normalize(data, args_im)
     # Make plot
     fig, ax = plt.subplots(figsize=args_im['figsize'])
+    # Make the heatmap
     _ = _heatmap(data, row_labels, col_labels, args_im, **args)
     # Add text into the cells
     # _  = _annotate_heatmap(im, valfmt="{x:.1f} t")
@@ -200,11 +413,11 @@ def seaborn(data, row_labels=None, col_labels=None, **args):
 
 
     """
-    assert not isinstance(data, pd.DataFrame), print('[IMAGESC] data input must be numpy array')
+    assert not isinstance(data, pd.DataFrame), print('[imagesc] >data input must be numpy array')
     try:
         import seaborn as sns
     except:
-        print('[IMAGESC] Error: seaborn is missing! Try to: pip install seaborn')
+        print('[imagesc] >Error: seaborn is missing! Try to: pip install seaborn')
 
     # Set defaults
     args, args_im = _defaults(args)
@@ -282,11 +495,11 @@ def cluster(data, row_labels=None, col_labels=None, **args):
 
 
     """
-    assert not isinstance(data, pd.DataFrame), print('[IMAGESC] data input must be numpy array')
+    assert not isinstance(data, pd.DataFrame), print('[imagesc] >data input must be numpy array')
     try:
         import seaborn as sns
     except:
-        print('[IMAGESC] Error: seaborn is missing! Try to: pip install seaborn')
+        print('[imagesc] >Error: seaborn is missing! Try to: pip install seaborn')
 
     # Set defaults
     args, args_im = _defaults(args)
@@ -345,7 +558,7 @@ def clean(data, row_labels=None, col_labels=None, **args):
 
 
     """
-    assert not isinstance(data, pd.DataFrame), print('[IMAGESC] data input must be numpy array')
+    assert not isinstance(data, pd.DataFrame), print('[imagesc] >data input must be numpy array')
     # Set defaults
     args, args_im = _defaults(args)
     # Linewidth if required
@@ -400,7 +613,7 @@ def fast(data, row_labels=None, col_labels=None, **args):
 
 
     """
-    assert not isinstance(data, pd.DataFrame), print('[IMAGESC] data input must be numpy array')
+    assert not isinstance(data, pd.DataFrame), print('[imagesc] >data input must be numpy array')
     # Set defaults
     args, args_im = _defaults(args)
     # Linewidth if required
@@ -515,7 +728,7 @@ def _heatmap(data, row_labels, col_labels, args_im, **args):
 
     if not ax:
         ax = plt.gca()
-        
+
     # Plot the heatmap
     im = ax.imshow(data, **args)
 
@@ -528,10 +741,10 @@ def _heatmap(data, row_labels, col_labels, args_im, **args):
         from mpl_toolkits.axes_grid1 import make_axes_locatable
         divider = make_axes_locatable(ax)
         cax = divider.append_axes("right", size="5%", pad=0.05)
-        plt.colorbar(im, cax=cax)
+        cbar = plt.colorbar(im, cax=cax)
 
     # We want to show all ticks...
-    if not col_labels is None:
+    if col_labels is not None:
         ax.set_xticks(np.arange(data.shape[1]))
         ax.set_xticklabels(col_labels)
         # Let the horizontal axes labeling appear on top.
@@ -543,7 +756,7 @@ def _heatmap(data, row_labels, col_labels, args_im, **args):
         # plt.setp(ax.get_xticklabels(), rotation=args_im['xtickRot'], ha="right", rotation_mode="anchor")
         ax.set_xticklabels(col_labels, rotation=args_im['xtickRot'], ha='center', minor=False)
 
-    if not row_labels is None:
+    if row_labels is not None:
         ax.set_yticks(np.arange(data.shape[0]))
         ax.set_yticklabels(row_labels)
         ax.set_yticklabels(row_labels, rotation=args_im['ytickRot'], ha='right', minor=False)
@@ -554,23 +767,24 @@ def _heatmap(data, row_labels, col_labels, args_im, **args):
 
     if args_im['grid']:
         # if not col_labels is None:
-        ax.set_xticks(np.arange(data.shape[1]+1)-.5, minor=True)
+        ax.set_xticks(np.arange(data.shape[1] + 1) - .5, minor=True)
         # if not row_labels is None:
-        ax.set_yticks(np.arange(data.shape[0]+1)-.5, minor=True)
+        ax.set_yticks(np.arange(data.shape[0] + 1) - .5, minor=True)
     if args_im['linewidth']>0 and args_im['grid']:
         ax.grid(which="minor", color=args_im['linecolor'], linestyle='-', linewidth=args_im['linewidth'])
-    if args_im['axis']==False:
+    if args_im['axis'] is False:
         plt.axis('off')
+    # Grid
     ax.grid(False)
 
     return(im)
 
 
-#%% Set defaults
+# %% Set defaults
 def _defaults(args):
     # Version check
     if not version.parse(matplotlib.__version__) > version.parse("3.1.1"):
-        print('[IMAGESC] Warning: Matplotlib version is advised to be to be > v3.1.1. Otherwise heatmaps can have cut-off tops and bottoms.\nTry to: pip install -U matplotlib')
+        print('[imagesc] >Warning: Matplotlib version is advised to be to be > v3.1.1. Otherwise heatmaps can have cut-off tops and bottoms.\nTry to: pip install -U matplotlib')
 
     # Extract the below for internal stuff
     getdefaults={'xlabel':None,'ylabel':None,'title':None,'axis':True,'grid':True,'normalize':False,'label_orientation':'below','verbose':3,'xtickRot':90,'ytickRot':0,'dpi':100,'figsize':(15,5)}
@@ -602,7 +816,7 @@ def _check_input(data, linewidth, args_im):
     if linewidth<0: linewidth=0
     # Check cols and rows with linewidth
     if ((data.shape[0]>100) or (data.shape[1]>100)) and (linewidth>0):
-        if args_im['verbose']>=2: print('[IMAGESC] WARNING: Plot will be poorly visible if [linewidth>0] with rows/columns>100. Set linewidth=0 to adjust. [auto-adjusting...]' )
+        if args_im['verbose']>=2: print('[imagesc] >WARNING: Plot will be poorly visible if [linewidth>0] with rows/columns>100. Set linewidth=0 to adjust. [auto-adjusting...]' )
         linewidth=0
     
     return(linewidth)
@@ -610,7 +824,7 @@ def _check_input(data, linewidth, args_im):
 # %% Check input
 def _normalize(data, args_im):
     if args_im['normalize']:
-        if args_im['verbose'] >=3: print('[IMAGESC] Normalzing data..')
+        if args_im['verbose'] >=3: print('[imagesc] >Normalzing data..')
         data = (data - data.mean()) / (data.max() - data.min())
     return(data)
 
@@ -622,6 +836,70 @@ def set_labels(data_shape, row_labels, col_labels):
         col_labels = np.arange(0, data_shape[1])
     return(row_labels, col_labels)
 
+# %% Adjust figure size based on input
+def _set_figsize(data_shape, figsize):
+    data_ratio = np.minimum(5/(data_shape[0]/data_shape[1]), 50)
+    out = tuple(np.ceil(np.interp(data_shape-np.min(data_shape), [np.min(figsize), data_ratio], [np.max(figsize), data_ratio])))
+    return(out[0], out[1])
+
+# %%
+def _path_check(path, verbose):
+    # Check wether path
+    if path is None:
+        path = os.path.join(tempfile.gettempdir(), 'index.html')
+    # Check wether dir + path
+    dirpath, filename = os.path.split(path)
+    # if input is single file, attach the absolute path.
+    if dirpath=='':
+        path = os.path.join(tempfile.gettempdir(), filename)
+        dirpath, filename = os.path.split(path)
+    # Check before proceeding
+    if (not '.html' in filename):
+        raise ValueError('[imagesc] >path should contain the file extension: ".html" ')
+    # Create dir
+    if not os.path.isdir(dirpath):
+        if verbose>=2: print('[imagesc] >Warning: Creating directory [%s]' %(dirpath))
+        os.makedirs(dirpath, exist_ok=True)
+    # Final
+    path = os.path.abspath(path)
+    dirpath, filename = os.path.split(path)
+    return filename, dirpath, path
+
+
+# %% Scaling
+def _scale(X, verbose=3):
+    """Scale data.
+
+    Description
+    -----------
+    Scaling in range by X*(100/max(X))
+
+    Parameters
+    ----------
+    X : array-like
+        Input image data.
+    verbose : int (default : 3)
+        Print to screen. 0: None, 1: Error, 2: Warning, 3: Info, 4: Debug, 5: Trace.
+
+    Returns
+    -------
+    df : array-like
+        Scaled image.
+
+    """
+    if verbose>=3: print('[imagesc] >Scaling image between [min-100]')
+    try:
+        # Normalizing between 0-100
+        # X = X - X.min()
+        X = X / X.max().max()
+        X = X * 100
+        X = np.round(X)
+    except:
+        if verbose>=2: print('[imagesc] >Warning: Scaling not possible.')
+
+    return X
+
+
 # %%
 def _annotate_heatmap(im, data=None, valfmt="{x:.2f}", textcolors=["black", "white"], threshold=None, **textkw):
     """Function to annotate a heatmap.
